@@ -1,4 +1,5 @@
-use bitflags::bitflags;
+//! Window managements, graphics and mouse controls.
+
 use static_assertions::assert_not_impl_all;
 use std::ptr::NonNull;
 
@@ -19,34 +20,14 @@ mod state;
 
 pub use border::*;
 pub use brightness::*;
-pub use builder::{WindowBuilder, WindowPos};
+use builder::WindowFlags;
+pub use builder::{WindowBuilder, WindowCoord};
 pub use config::*;
 pub use grab::*;
 pub use hit_test::*;
 pub use state::*;
 
-bitflags! {
-    struct WindowFlags: u32 {
-        const FULLSCREEN = bind::SDL_WindowFlags_SDL_WINDOW_FULLSCREEN;
-        const FULLSCREEN_DESKTOP = bind::SDL_WindowFlags_SDL_WINDOW_FULLSCREEN_DESKTOP;
-        const OPENGL = bind::SDL_WindowFlags_SDL_WINDOW_OPENGL;
-        const VULKAN = bind::SDL_WindowFlags_SDL_WINDOW_VULKAN;
-        const METAL = bind::SDL_WindowFlags_SDL_WINDOW_METAL;
-        const SHOWN = bind::SDL_WindowFlags_SDL_WINDOW_SHOWN;
-        const HIDDEN = bind::SDL_WindowFlags_SDL_WINDOW_HIDDEN;
-        const BORDERLESS = bind::SDL_WindowFlags_SDL_WINDOW_BORDERLESS;
-        const RESIZABLE = bind::SDL_WindowFlags_SDL_WINDOW_RESIZABLE;
-        const MINIMIZED = bind::SDL_WindowFlags_SDL_WINDOW_MINIMIZED;
-        const MAXIMIZED = bind::SDL_WindowFlags_SDL_WINDOW_MAXIMIZED;
-        const INPUT_GRABBED = bind::SDL_WindowFlags_SDL_WINDOW_INPUT_GRABBED;
-        const INPUT_FOCUS = bind::SDL_WindowFlags_SDL_WINDOW_INPUT_FOCUS;
-        const MOUSE_FOCUS = bind::SDL_WindowFlags_SDL_WINDOW_MOUSE_FOCUS;
-        const FOREIGN = bind::SDL_WindowFlags_SDL_WINDOW_FOREIGN;
-        const ALLOW_HIGHDPI = bind::SDL_WindowFlags_SDL_WINDOW_ALLOW_HIGHDPI;
-        const MOUSE_CAPTURE = bind::SDL_WindowFlags_SDL_WINDOW_MOUSE_CAPTURE;
-    }
-}
-
+/// A window made by SDL2.
 pub struct Window<'video> {
     window: NonNull<bind::SDL_Window>,
     video: &'video Video<'video>,
@@ -61,55 +42,66 @@ impl std::fmt::Debug for Window<'_> {
 assert_not_impl_all!(Window: Send, Sync);
 
 impl<'video> Window<'video> {
+    /// Gets a window from the window id, or `None` if does not exist.
     pub fn from_id(id: u32, video: &'video Video) -> Option<Self> {
         let raw = unsafe { bind::SDL_GetWindowFromID(id) };
         NonNull::new(raw).map(|window| Self { window, video })
     }
 
+    /// Gets a grabbed window, or `None` if does not exist.
     pub fn grabbed(video: &'video Video) -> Option<Self> {
         let raw = unsafe { bind::SDL_GetGrabbedWindow() };
         NonNull::new(raw).map(|window| Self { window, video })
     }
 
+    /// Gets a focused window, or `None` if does not exist.
     pub fn mouse_focused(video: &'video Video) -> Option<Self> {
         let raw = unsafe { bind::SDL_GetMouseFocus() };
         NonNull::new(raw).map(|window| Self { window, video })
     }
 
-    pub fn as_ptr(&self) -> *mut bind::SDL_Window {
+    pub(crate) fn as_ptr(&self) -> *mut bind::SDL_Window {
         self.window.as_ptr()
     }
 
+    /// Returns the state of the window.
     pub fn state(&self) -> WindowState {
         let flag_bits = unsafe { bind::SDL_GetWindowFlags(self.as_ptr()) };
         WindowFlags::from_bits_truncate(flag_bits).into()
     }
 
+    /// Returns the display at the window, or `None` if unavailable.
     pub fn display(&self) -> Option<Display> {
         let ret = unsafe { bind::SDL_GetWindowDisplayIndex(self.as_ptr()) };
         (0 <= ret).then(|| Display::new(ret, self.video))
     }
 
+    /// Returns the window id.
     pub fn id(&self) -> u32 {
         unsafe { bind::SDL_GetWindowID(self.as_ptr()) }
     }
 
+    /// Returns the pixel format of the window context.
     pub fn pixel_format(&self) -> PixelFormatKind {
         unsafe { bind::SDL_GetWindowPixelFormat(self.as_ptr()) }.into()
     }
 
+    /// Shows the window.
     pub fn show(&self) {
         unsafe { bind::SDL_ShowWindow(self.as_ptr()) }
     }
 
+    /// Hides the window.
     pub fn hide(&self) {
         unsafe { bind::SDL_HideWindow(self.as_ptr()) }
     }
 
+    /// Raises the window.
     pub fn raise(&self) {
         unsafe { bind::SDL_RaiseWindow(self.as_ptr()) }
     }
 
+    /// Make the window full screen, or `Err` on failure.
     pub fn full_screen(&self) -> Result<()> {
         let ret = unsafe {
             bind::SDL_SetWindowFullscreen(
@@ -123,26 +115,32 @@ impl<'video> Window<'video> {
         Ok(())
     }
 
+    /// Maximizes the window.
     pub fn maximize(&self) {
         unsafe { bind::SDL_MaximizeWindow(self.as_ptr()) }
     }
 
+    /// Minimizes the window.
     pub fn minimize(&self) {
         unsafe { bind::SDL_MinimizeWindow(self.as_ptr()) }
     }
 
+    /// Restores the window from maximization/minimization.
     pub fn restore(&self) {
         unsafe { bind::SDL_RestoreWindow(self.as_ptr()) }
     }
 
+    /// Sets an icon from a surface for the window.
     pub fn set_icon(&self, icon: &impl Surface) {
         unsafe { bind::SDL_SetWindowIcon(self.as_ptr(), icon.as_ptr().as_ptr()) }
     }
 
+    /// Returns whether the window is showing the screen keyboard.
     pub fn is_screen_keyboard_shown(&self) -> bool {
         unsafe { bind::SDL_IsScreenKeyboardShown(self.as_ptr()) != 0 }
     }
 
+    /// Makes the window surface.
     pub fn surface(&self) -> WindowSurface {
         WindowSurface::new(self)
     }
